extern "C"
{
#include "user_interface.h"  	  // Required for wifi_station_connect() to work
}

#include <ESP8266WiFi.h>          // https://github.com/esp8266/Arduino
#include <WiFiUdp.h>
#include <time.h>

//needed for library
#include <DNSServer.h>
#include <ESP8266WebServer.h>
#include <WiFiManager.h>          // https://github.com/tzapu/WiFiManager
#include <NTPClient.h>			  // https://github.com/arduino-libraries/NTPClient
#include <Timezone.h>    		  // https://github.com/JChristensen/Timezone

WiFiUDP ntpUDP;

// By default 'pool.ntp.org' is used with 60 seconds update interval and
// no offset
NTPClient timeClient(ntpUDP);

// You can specify the time server pool and the offset, (in seconds)
// additionaly you can specify the update interval (in milliseconds).
// NTPClient timeClient(ntpUDP, "europe.pool.ntp.org", 3600, 60000);

WiFiManager wifiManager;

//int firstrun = 1;

// configure your timezone rules here how described on https://github.com/JChristensen/Timezone
TimeChangeRule myDST =
{ "UTC", Last, Sun, Mar, 2, 0 };    //Daylight time = UTC + 0 hours
TimeChangeRule mySTD =
{ "UTC", Last, Sun, Oct, 2, 0 };      //Standard time = UTC + 0 hours
Timezone myTZ(myDST, mySTD);

void setup()
{
	pinMode(BUILTIN_LED, OUTPUT);   // Initialize the BUILTIN_LED pin as an output
	digitalWrite(BUILTIN_LED, HIGH);
	Serial.begin(115200);

	wifiManager.setTimeout(180);

	//fetches ssid and pass and tries to connect
	//if it does not connect it starts an access point with the specified name
	//here  "NixieAP"
	//and goes into a blocking loop awaiting configuration

	if (!wifiManager.autoConnect("NixieAP"))
	{
		Serial.println("failed to connect and hit timeout");
		delay(3000);
		//reset and try again, or maybe put it to deep sleep
		ESP.reset();
		delay(5000);
	}

	//if you get here you have connected to the WiFi
	Serial.println("connected...yeey :)");

	Serial1.begin(9600);

	timeClient.update();
}

void loop()
{
	char tstr[128];
	unsigned char cs;
	unsigned int i;
	time_t rawtime, loctime;
	unsigned char curr_sec=0, last_sec=0, has_fix='V';

	for (;;)
	{
		if (timeClient.update())
      has_fix='A';
    else
      has_fix='V';
		
		rawtime = timeClient.getEpochTime();		// get NTP-time
		loctime = myTZ.toLocal(rawtime);			// calc local time

    curr_sec = second(loctime);
    
		if(curr_sec != last_sec)
		{
			digitalWrite(BUILTIN_LED, HIGH);		// blink for sync
			sprintf(tstr, "$GPRMC,%02d%02d%02d,%c,0000.0000,N,00000.0000,E,0.0,0.0,%02d%02d%02d,0.0,E,S",
					hour(loctime), minute(loctime), second(loctime), has_fix, day(loctime), month(loctime), year(loctime) - 2000);
			cs = 0;
			for (i = 1; i < strlen(tstr); i++)		// calculate checksum
				cs ^= tstr[i];
			sprintf(tstr + strlen(tstr), "*%02X", cs);
			Serial.println(tstr);					// send to console
			Serial1.println(tstr);					// send to clock
			delay(100);
			digitalWrite(BUILTIN_LED, LOW);
      last_sec = curr_sec;
		}
	}
}
